package vacworld;

import agent.*;

/** A simulator for the vacuum cleaning world environment. */
public class VacuumWorld extends Environment{

  public VacuumWorld() {
  }

  /** Add a new agent to the environment. Since the vacuum cleaning
      world is a single agent environment, this method should only
      be called once per object. */
  public void addAgent(Agent agent) {

    if (agents.isEmpty() == false) {
      System.out.println("ERROR - tried to add a second agent to a single agent environment");
    }
    else {
      agents.add(agent);
    }
  }
 

  /** The simulation is complete when the robot has performed its ShutOff
      action. */
  protected boolean isComplete() {

    if (((VacuumState)state).isRobotOff())
      return true;
    else
      return false;
  }

  
  /** Create a percept for an agent. This implements the see: S -> P
      function. */
  protected Percept getPercept(Agent a) {
    
    VacPercept p;

    if (state instanceof VacuumState) {
      p = new VacPercept((VacuumState)state, a);
      System.out.println("Pecept: " + p.toString());
      return p;
    }
    else {
      System.out.println("ERROR - state is not a VacuumState object.");
      return null;
    }
  }

  
  /** Execute an agent's action and update the environment's state. */
  protected void updateState(Agent a, Action action) {

    super.updateState(a, action);
    System.out.println("Action: " + action.toString());
    System.out.println();
  }


  /** Starts the program. */
  public static void main(String[] args) {

    VacuumWorld world;
    VacAgent agent;
    VacuumState initState;

    System.out.println("The Vacuum Cleaner World Agent Testbed");
    System.out.println("--------------------------------------");
    System.out.println();

    world = new VacuumWorld();
    agent = new VacAgent();
    initState = VacuumState.getInitState();
    // initState = VacuumState.getRandomState();
    world.addAgent(agent);
    world.start(initState);
  }

}
